﻿using ComputerShopClientApp.Models;
using ComputerShopContracts.BindingModels;
using ComputerShopContracts.SearchModels;
using ComputerShopContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using System.Diagnostics;

namespace ComputerShopClientApp.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public HomeController(ILogger<HomeController> logger)
        {
            _logger = logger;
        }

        public IActionResult Index()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<PurchaseViewModel>>($"api/main/getpurchases?clientId={APIClient.Client.Id}"));
        }

        public IActionResult Component()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<ComponentViewModel>>($"api/component/getcomponentlist?clientId={APIClient.Client.Id}"));
        }
        public IActionResult Assembly()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<AssemblyViewModel>>($"api/Assembly/getassemblylist?clientId={APIClient.Client.Id}"));
        }

        public IActionResult Report()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View();
        }

        public IActionResult ReportDocXml()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<ComponentViewModel>>($"api/Component/getcomponentlist?clientId={APIClient.Client.Id}"));
        }

        [HttpGet]
        public IActionResult Privacy()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.Client);
        }

        [HttpPost]
        public void Privacy(string login, string password, string fio)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/updatedata", new ClientBindingModel
            {
                Id = APIClient.Client.Id,
                ClientFIO = fio,
                Email = login,
                Password = password
            });

            APIClient.Client.ClientFIO = fio;
            APIClient.Client.Email = login;
            APIClient.Client.Password = password;
            Response.Redirect("Index");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }

        [HttpPost]
        public void ReportDocXml(int[] Ids, string type)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }

            if (Ids.Length <= 0)
            {
                throw new Exception("Количество должно быть больше 0");
            }

            if (string.IsNullOrEmpty(type))
            {
                throw new Exception("Неверный тип отчета");
            }

            List<int> res = new List<int>();

            foreach (var item in Ids)
            {
                res.Add(item);
            }

            if (type == "docx")
            {
                APIClient.PostRequest("api/report/createdocreport", new ReportBindingModel
                {
                    Ids = res,
                    FileName = "F:\\ReportsCourseWork\\wordfile.docx"
                });
                Response.Redirect("GetWordFile");
            }
            else
            {
                APIClient.PostRequest("api/report/createxmlreport", new ReportBindingModel
                {
                    Ids = res,
                    FileName = "F:\\ReportsCourseWork\\excelfile.xlsx"
                });
                Response.Redirect("GetExcelFile");
            }
        }

        [HttpGet]
        public IActionResult GetWordFile()
        {
            return new PhysicalFileResult("F:\\ReportsCourseWork\\wordfile.docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        }

        public IActionResult GetExcelFile()
        {
            return new PhysicalFileResult("F:\\ReportsCourseWork\\excelfile.xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        }

        [HttpGet]
        public IActionResult Enter()
        {
            return View();
        }

        [HttpPost]
        public void Enter(string login, string password)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password))
            {
                throw new Exception("Введите логин и пароль");
            }
            APIClient.Client = APIClient.GetRequest<ClientViewModel>($"api/client/login?login={login}&password={password}");
            if (APIClient.Client == null)
            {
                throw new Exception("Неверный логин/пароль");
            }
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Register()
        {
            return View();
        }

        [HttpPost]
        public void Register(string login, string password, string fio)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/register", new ClientBindingModel
            {
                ClientFIO = fio,
                Email = login,
                Password = password
            });
            Response.Redirect("Enter");
            return;
        }

        [HttpGet]
        public IActionResult CreatePurchase()
        {
            ViewBag.Components = APIClient.GetRequest<List<ComponentViewModel>>($"api/main/getcomponentlist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void CreatePurchase(int component, int count)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (count <= 0)
            {
                throw new Exception("Количество и сумма должны быть больше 0");
            }
            APIClient.PostRequest("api/main/createpurchase", new PurchaseBindingModel
            {
                ClientId = APIClient.Client.Id,
                ComponentId = component,
                Count = count,
                Sum = Calc(count, component)
            });
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult CreateComponent()
        {
            return View();
        }
        [HttpPost]
        public void CreateComponent(string name, int cost)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (cost <= 0)
            {
                throw new Exception("Цена должна быть больше 0");
            }
            APIClient.PostRequest("api/component/createcomponent", new ComponentBindingModel
            {
                ClientId = APIClient.Client.Id,
                ComponentName = name,
                Cost = cost
            });
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult DeleteComponent()
        {
            ViewBag.Components = APIClient.GetRequest<List<ComponentViewModel>>($"api/main/getcomponentlist?clientId={APIClient.Client.Id}");
            return View();
        }
        [HttpPost]
        public void DeleteComponent(int component)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            APIClient.PostRequest("api/component/deletecomponent", new ComponentBindingModel
            {
                Id = component
            });
            Response.Redirect("Component");
        }

        [HttpGet]
        public IActionResult CreateAssembly()
        {
            ViewBag.Components = APIClient.GetRequest<List<ComponentViewModel>>($"api/component/getcomponentlist?clientId={APIClient.Client.Id}");
            ViewBag.CurrentComponents = new List<ComponentViewModel>();
            return View();
        }

        [HttpPost]
        public void CreateAssembly(string name, int cost)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            APIClient.PostRequest("api/assembly/createassembly", new AssemblyBindingModel
            {
                ClientId = APIClient.Client.Id,
                AssemblyName = name,
                Price = cost,
            });
            System.Diagnostics.Debug.WriteLine("it might work");
            Response.Redirect("Assembly");
        }

        [HttpGet]
        public IActionResult DeleteAssembly()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Assemblies = APIClient.GetRequest<List<AssemblyViewModel>>($"api/assembly/GetAssemblyList?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void DeleteAssembly(int assembly)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            APIClient.PostRequest("api/assembly/deleteassembly", new AssemblyBindingModel
            {
                Id = assembly
            });
            Response.Redirect("Assembly");
        }

        public IActionResult AddComponentToAssembly()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Assemblies = APIClient.GetRequest<List<AssemblyViewModel>>($"api/assembly/getassemblylist?clientId={APIClient.Client.Id}");
            ViewBag.Components = APIClient.GetRequest<List<ComponentViewModel>>($"api/component/getcomponentlist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void AddComponentToAssembly(int assembly, int component, int amount)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            APIClient.PostRequest("api/assembly/AddComponentToAssembly", Tuple.Create(
                new AssemblySearchModel() { Id = assembly },
                new ComponentSearchModel() { Id = component },
                amount
            ));
            Response.Redirect("Assembly");
        }

        public IActionResult EditAssembly()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Assemblies = APIClient.GetRequest<List<AssemblyViewModel>>($"api/assembly/getassemblylist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void EditAssembly(int assembly, string assemblyName, int price)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            if (string.IsNullOrEmpty(assemblyName))
            {
                throw new Exception("Название не может быть пустым");
            }
            if (price < 0)
            {
                throw new Exception("Цена не может быть меньше нуля");
            }
            APIClient.PostRequest("api/assembly/editassembly", new AssemblyBindingModel
            {
                Id = assembly,
                AssemblyName = assemblyName,
                Price = price,
                ClientId = APIClient.Client.Id
            });
            Response.Redirect("Assembly");
        }

        [HttpGet]
        public Tuple<AssemblyViewModel, string>? GetAssembly(int assemblyId)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            var result = APIClient.GetRequest<Tuple<AssemblyViewModel, List<Tuple<string, int>>>>($"api/assembly/getassembly?assemblyId={assemblyId}");
            if (result == null)
            {
                return default;
            }
            string table = "";
            for (int i = 0; i < result.Item2.Count; i++)
            {
                var componentName = result.Item2[i].Item1;
                var componentAmount = result.Item2[i].Item2;
                table += "<tr style=\"height: 44px\">";
                table += $"<td class=\"u-border-1 u-border-grey-30 u-table-cell\">{componentName}</td>";
                table += $"<td class=\"u-border-1 u-border-grey-30 u-table-cell\">{componentAmount}</td>";
                table += "</tr>";
            }
            return Tuple.Create(result.Item1, table);
        }



        [HttpPost]
        public double Calc(int count, int component)
        {
            var comp = APIClient.GetRequest<ComponentViewModel>($"api/main/getcomponent?componentId={component}");
            return count * (comp?.Cost ?? 1);
        }

        public IActionResult EditComponent()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Components = APIClient.GetRequest<List<ComponentViewModel>>($"api/component/getcomponentlist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void EditComponent(int component, string componentName, int price)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            if (string.IsNullOrEmpty(componentName))
            {
                throw new Exception("Название не может быть пустым");
            }
            if (price < 0)
            {
                throw new Exception("Цена не может быть меньше нуля");
            }
            APIClient.PostRequest("api/component/editcomponent", new ComponentBindingModel
            {
                Id = component,
                ComponentName = componentName,
                Cost = price,
                ClientId = APIClient.Client.Id
            });
            Response.Redirect("Assembly");
        }

        [HttpGet]
        public ComponentViewModel? GetComponent(int componentId)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            var result = APIClient.GetRequest<ComponentViewModel>($"api/component/getcomponent?componentId={componentId}");
            if (result == null)
            {
                return default;
            }
            return result;
        }
    }
}