﻿using ComputerShopContracts.BindingModels;
using ComputerShopContracts.ViewModels;
using ComputerShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerShopDatabaseImplement.Models
{
    internal class Component : IComponentModel
    {
        public int Id { get; private set; }
        [Required]
        public string ComponentName { get; private set; } = string.Empty;
        [Required]
        public double Cost { get; set; }
        public int ClientId { get; private set; }
        [ForeignKey("ComponentId")]
        public virtual List<AssemblyComponent> AssemblyComponents { get; set; } =
        new();
        [ForeignKey("ComponentId")]
        public virtual List<Purchase> Purchases { get; set; } = new();
        [ForeignKey("ComponentId")]
        public virtual List<ComponentSupply> Supplies { get; set; } = new();
        public static Component? Create(ComponentBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Component()
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost,
                ClientId = model.ClientId
            };
        }
        public static Component Create(ComponentViewModel model)
        {
            return new Component
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost,
                ClientId = model.ClientId
            };
        }
        public void Update(ComponentBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ComponentName = model.ComponentName;
            Cost = model.Cost;
            ClientId = model.ClientId;
        }
        public ComponentViewModel GetViewModel => new()
        {
            Id = Id,
            ComponentName = ComponentName,
            Cost = Cost,
            ClientId = ClientId
        };
    }
}
