﻿using ComputerShopContracts.BindingModels;
using ComputerShopContracts.BusinessLogicContracts;
using ComputerShopContracts.SearchModels;
using ComputerShopContracts.ViewModels;
using ComputerShopDataModels.Models;
using Microsoft.AspNetCore.Mvc;

namespace ComputerShopRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class AssemblyController : Controller
	{
        private readonly ILogger _logger;
        private readonly IAssemblyLogic _assembly;
        private readonly IComponentLogic _component;
        public AssemblyController(ILogger<AssemblyController> logger, IAssemblyLogic assembly, IComponentLogic component)
        {
            _logger = logger;
            _assembly = assembly;
            _component = component;
        }
        [HttpGet]
        public List<AssemblyViewModel>? GetAssemblyList(int clientId)
        {
            try
            {
                return _assembly.ReadList(new AssemblySearchModel
                {
                    ClientId = clientId,
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка сборок");
                throw;
            }
        }

        [HttpGet]
        public Tuple<AssemblyViewModel, List<Tuple<string, int>>>? GetAssembly(int assemblyId)
        {
            try
            {
                var elem = _assembly.ReadElement(new AssemblySearchModel { Id = assemblyId });
                if (elem == null)
                    return null;
                return Tuple.Create(elem, elem.AssemblyComponents.Select(x => Tuple.Create(x.Value.Item1.ComponentName, x.Value.Item2)).ToList());
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения сборки по id={Id}", assemblyId);
                throw;
            }
        }

        [HttpGet]
        public List<ComponentViewModel>? GetComponentList()
        {
            try
            {
                return _component.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка компонентов");
                throw;
            }
        }
        [HttpPost]
        public void CreateAssembly(AssemblyBindingModel model)
        {
            try
            {
                _assembly.Create(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания сборки");
                throw;
            }
        }
        [HttpPost]
        public void DeleteAssembly(AssemblyBindingModel model)
        {
            try
            {
                _assembly.Delete(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка удаления сборки");
                throw;
            }
        }

        [HttpPost]
        public void AddComponentToAssembly(Tuple<AssemblySearchModel, ComponentSearchModel, int> model)
        {
            try
            {
                _assembly.AddComponentToAssembly(model.Item1, model.Item2, model.Item3);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка добавления компонента в сборку.");
                throw;
            }
        }

        [HttpPost]
        public void EditAssembly(AssemblyBindingModel model)
        {
            try
            {
                _assembly.Update(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка удаления сборки");
                throw;
            }
        }
    }
}
